@@ -14,20 +14,21 @@
 
 import warnings
 
+from keystoneauth1 import adapter
 from keystoneauth1 import plugin
 from keystoneauth1 import session
+import requestsexceptions
 
 from os_client_config import _log
 from os_client_config import exceptions
 
-# Importing these for later but not disabling for now
-try:
-    from requests.packages.urllib3 import exceptions as urllib_exc
-except ImportError:
-    try:
-        from urllib3 import exceptions as urllib_exc
-    except ImportError:
-        urllib_exc = None
+
+def _make_key(key, service_type):
+    if not service_type:
+        return key
+    else:
+        service_type = service_type.lower().replace('-', '_')
+        return "_".join([service_type, key])
 
 
 class CloudConfig(object):
@@ -96,32 +97,30 @@ class CloudConfig(object):
         return self.config['auth']
 
     def get_interface(self, service_type=None):
+        key = _make_key('interface', service_type)
         interface = self.config.get('interface')
-        if not service_type:
-            return interface
-        key = '{service_type}_interface'.format(service_type=service_type)
         return self.config.get(key, interface)
 
     def get_region_name(self, service_type=None):
         if not service_type:
             return self.region
-        key = '{service_type}_region_name'.format(service_type=service_type)
+        key = _make_key('region_name', service_type)
         return self.config.get(key, self.region)
 
     def get_api_version(self, service_type):
-        key = '{service_type}_api_version'.format(service_type=service_type)
+        key = _make_key('api_version', service_type)
         return self.config.get(key, None)
 
     def get_service_type(self, service_type):
-        key = '{service_type}_service_type'.format(service_type=service_type)
+        key = _make_key('service_type', service_type)
         return self.config.get(key, service_type)
 
     def get_service_name(self, service_type):
-        key = '{service_type}_service_name'.format(service_type=service_type)
+        key = _make_key('service_name', service_type)
         return self.config.get(key, None)
 
     def get_endpoint(self, service_type):
-        key = '{service_type}_endpoint'.format(service_type=service_type)
+        key = _make_key('endpoint', service_type)
         return self.config.get(key, None)
 
     @property
@@ -146,13 +145,12 @@ class CloudConfig(object):
             # Turn off urllib3 warnings about insecure certs if we have
             # explicitly configured requests to tell it we do not want
             # cert verification
-            if not verify and urllib_exc is not None:
+            if not verify:
                 self.log.debug(
                     "Turning off SSL warnings for {cloud}:{region}"
                     " since verify=False".format(
                         cloud=self.name, region=self.region))
-                warnings.filterwarnings(
-                    'ignore', category=urllib_exc.InsecureRequestWarning)
+            requestsexceptions.squelch_warnings(insecure_requests=not verify)
             self._keystone_session = session.Session(
                 auth=self._auth,
                 verify=verify,
@@ -160,6 +158,28 @@ class CloudConfig(object):
                 timeout=self.config['api_timeout'])
         return self._keystone_session
 
+    def get_session_client(self, service_key):
+        """Return a prepped requests adapter for a given service.
+
+        This is useful for making direct requests calls against a
+        'mounted' endpoint. That is, if you do:
+
+          client = get_session_client('compute')
+
+        then you can do:
+
+          client.get('/flavors')
+
+        and it will work like you think.
+        """
+
+        return adapter.Adapter(
+            session=self.get_session(),
+            service_type=self.get_service_type(service_key),
+            service_name=self.get_service_name(service_key),
+            interface=self.get_interface(service_key),
+            region_name=self.region)
+
     def get_session_endpoint(self, service_key):
         """Return the endpoint from config or the catalog.
 
@@ -263,15 +283,11 @@ class CloudConfig(object):
             if service_key == 'network' and version == '2':
                 version = '2.0'
             if service_key == 'identity':
-                # keystoneclient takes version as a tuple.
-                version = tuple(str(float(version)).split('.'))
-                constructor_kwargs['version'] = version
                 # Workaround for bug#1513839
                 if 'endpoint' not in constructor_kwargs:
                     endpoint = self.get_session_endpoint('identity')
                     constructor_kwargs['endpoint'] = endpoint
-            else:
-                constructor_args.append(version)
+            constructor_args.append(version)
 
         return client_class(*constructor_args, **constructor_kwargs)
 
@@ -281,7 +297,7 @@ class CloudConfig(object):
         endpoint = self.get_session_endpoint(service_key='object-store')
         if not endpoint:
             return None
-        return client_class(
+        swift_kwargs = dict(
             preauthurl=endpoint,
             preauthtoken=token,
             auth_version=self.get_api_version('identity'),
@@ -289,8 +305,10 @@ class CloudConfig(object):
                 auth_token=token,
                 object_storage_url=endpoint,
                 region_name=self.get_region_name()),
-            timeout=self.api_timeout,
         )
+        if self.config['api_timeout'] is not None:
+            swift_kwargs['timeout'] = float(self.config['api_timeout'])
+        return client_class(**swift_kwargs)
 
     def get_cache_expiration_time(self):
         if self._openstack_config:
